from gurobipy import *
import pandas as pd
class MDDF():
    def __init__(self, data: pd.DataFrame, input_cols: list, output_cols: list, un_output_cols: list, vv, dd):
        self.data = data
        self.K = list(range(0,data.shape[0]))
        self.input = input_cols
        self.output = output_cols
        self.unoutput = un_output_cols
        self.key = vv
        self.dd = dd
        
    def res(self):
        TE = []
        for i in self.K:
            eff = self.__DDFmodel(i,self.data)
            TE.append(eff)
        self.data['DDF效率'] = TE
        return self.data[[self.data.columns.values[0]]+self.input+self.output+self.unoutput+['DDF效率']]
    
    def __DDFmodel(self, i,data):
        model = Model('DDF')
        model.Params.LogToConsole = 0
        lambdas = model.addVars(self.K, lb=0, vtype=GRB.CONTINUOUS, name='lambdas')
        beta = model.addVar(vtype = GRB.CONTINUOUS, name = 'beta')
        model.addConstrs(quicksum(lambdas[j] * data.loc[j,p] for j in self.K) 
                         <= (1-beta) * data.loc[i,p]  for p in self.input)
        model.addConstrs(quicksum(lambdas[j] * data.loc[j,q] for j in self.K) 
                         >= (1+beta) * data.loc[i,q]  for q in self.output)
        if self.unoutput!= []:
            if self.dd == '(-x,y,-b)弱可处置性=':
                    model.addConstrs(quicksum(lambdas[j] * data.loc[j,u] for j in self.K) 
                         == (1-beta) * data.loc[i,u]  for u in self.unoutput)
            elif self.dd == '(-x,y,-b)强可处置性>=':
                    model.addConstrs(quicksum(lambdas[j] * data.loc[j,u] for j in self.K) 
                         >= (1-beta) * data.loc[i,u]  for u in self.unoutput)
            elif self.dd == '(-x,y,b)强可处置性<=':
                    model.addConstrs(quicksum(lambdas[j] * data.loc[j,u] for j in self.K) 
                         <= (1+beta) * data.loc[i,u]  for u in self.unoutput)
        if self.key == '规模报酬可变VRS':
            model.addConstr(quicksum(lambdas[j] for j in self.K) == 1)
        model.setObjective(beta, GRB.MAXIMIZE)
        model.optimize()
        eff = model.getObjective().getValue() if model.status == gurobipy.GRB.Status.OPTIMAL else 'N/A'
        return eff


from pyomo.environ import *  
import pandas as pd

class MDDF_pyomo():
    def __init__(self, data: pd.DataFrame, input_cols: list, output_cols: list, un_output_cols: list, vv, dd,solver):
        self.data = data
        self.K = list(range(0,data.shape[0]))
        self.input = input_cols
        self.output = output_cols
        self.unoutput = un_output_cols
        self.key = vv
        self.dd = dd
        self.solvers = solver
        
    def res(self):
        TE = []
        for i in self.K:
            eff = self.__DDFmodel(i,self.data)
            TE.append(eff)
        self.data['DDF效率'] = TE
        return self.data[[self.data.columns.values[0]]+self.input+self.output+self.unoutput+['DDF效率']]
    
    def __DDFmodel(self, i,data):
        model = ConcreteModel()
        model.lambdas = Var(self.K, within=NonNegativeReals)
        model.beta = Var(within=NonNegativeReals)
        model.constraints = ConstraintList()
        for p in self.input:
            model.constraints.add(expr=sum(model.lambdas[j] * data.loc[j,p] for j in self.K) 
                            <= (1-model.beta) * data.loc[i,p])
        for q in self.output:
            model.constraints.add(expr=sum(model.lambdas[j] * data.loc[j,q] for j in self.K) 
                            >= (1+model.beta) * data.loc[i,q] )
        if self.unoutput!= []:
            if self.dd == '(-x,y,-b)弱可处置性=':
                for u in self.unoutput:
                    model.constraints.add(expr=sum(model.lambdas[j] * data.loc[j,u] for j in self.K) 
                         == (1-model.beta) * data.loc[i,u])
            elif self.dd == '(-x,y,-b)强可处置性>=':
                for u in self.unoutput:
                    model.constraints.add(expr=sum(model.lambdas[j] * data.loc[j,u] for j in self.K) 
                         >= (1-model.beta) * data.loc[i,u])
            elif self.dd == '(-x,y,b)强可处置性<=':
                for u in self.unoutput:
                    model.constraints.add(expr=sum(model.lambdas[j] * data.loc[j,u] for j in self.K) 
                         <= (1+model.beta) * data.loc[i,u])
        if self.key == '规模报酬可变VRS':
            model.constraints.add(expr=sum(model.lambdas[j] for j in self.K) == 1)
        model.obj = Objective(expr=model.beta, sense=maximize)
        solver = SolverFactory(self.solvers)
        opt = solver.solve(model)
        eff = value(model.obj)
        return eff


from pyomo.environ import *  
import pandas as pd
class MCCR_pyomo():
    def __init__(self, data: pd.DataFrame, input_cols: list, output_cols: list,un_output_cols: list, vv, solver):
        self.data = data
        self.K = list(range(0,data.shape[0]))
        self.input = input_cols+un_output_cols
        self.output = output_cols
        self.solvers = solver

    def res(self):
        TE = []
        PTE = []
        SE = []
        Type = []
        for i in self.K:
            eff_C, Types = self.__CCRmodel(i)
            eff_B = self.__BCCmodel(i)
            eff_S = eff_C/eff_B
            TE.append(eff_C)
            PTE.append(eff_B)
            SE.append(eff_S)
            Type.append(Types)
        self.data['TE'] = TE
        self.data['PTE'] = PTE
        self.data['SE'] = SE
        self.data['规模报酬分析'] = Type
        return self.data[self.input+self.output+['TE','PTE','SE','规模报酬分析']]
    
    def __CCRmodel(self, i):
        model = ConcreteModel()
        model.lambdas = Var(self.K, within=NonNegativeReals)
        model.sita = Var(within=NonNegativeReals)
        model.constraints = ConstraintList()
        for p in self.output:
            model.constraints.add(expr=sum(model.lambdas[j] * self.data.loc[j,p] for j in self.K) - self.data.loc[i,p] >=0)
        for q in self.input:
            model.constraints.add(expr=sum(model.lambdas[j] * self.data.loc[j,q] for j in self.K) - model.sita * self.data.loc[i,q] <=0)
        model.obj = Objective(expr=model.sita, sense=minimize)
        solver = SolverFactory(self.solvers)
        opt = solver.solve(model)
        eff = value(model.sita)
        Type = '规模报酬固定' if sum(value(model.lambdas[j]) for j in self.K) == 1 else '规模报酬递增' if sum(value(model.lambdas[j]) for j in self.K)  < 1 else '规模报酬递减'
        return eff, Type
    
    def __BCCmodel(self, i):
        model = ConcreteModel()
        model.lambdas = Var(self.K, within=NonNegativeReals)
        model.sita = Var(within=NonNegativeReals)
        model.constraints = ConstraintList()
        for p in self.output:
            model.constraints.add(expr=sum(model.lambdas[j] * self.data.loc[j,p] for j in self.K) - self.data.loc[i,p] >=0)
        for q in self.input:
            model.constraints.add(expr=sum(model.lambdas[j] * self.data.loc[j,q] for j in self.K) - model.sita * self.data.loc[i,q] <=0)
        model.constraints.add(expr=sum(model.lambdas[j] for j in self.K) == 1)
        model.obj = Objective(expr=model.sita, sense=minimize)
        solver = SolverFactory(self.solvers)
        opt = solver.solve(model)
        eff = value(model.sita)
        return eff

if __name__ == '__main__':
    data = pd.read_excel(r'C:\Users\wangy\Desktop\测试数据.xlsx')
    input_cols = ['输入1','输入2']
    output_cols = ['输出1','输出2']
    un_output_cols = ['非期望输出1']
    model = MCCR_pyomo(data=data, input_cols = input_cols, output_cols = output_cols,un_output_cols = un_output_cols, vv = 1, solver = 'cplex' )
    print(model.res())


from PySide2 import QtCore
from PySide2.QtGui import QIcon
from PySide2.QtWidgets import *
from deahub.CCR import MCCR
from deahub.DDF import MDDF
from deahub.SBM import MSBM
from deahub.CCR_pyomo import MCCR_pyomo
from deahub.DDF_pyomo import MDDF_pyomo
from deahub.SBM_pyomo import MSBM_pyomo
from deahub.Ui_table import Ui_Dialog
import pandas as pd
import os

class Tables(QDialog, Ui_Dialog):
    def __init__(self):
        super().__init__()
        self.setupUi(self)
        self.radioButton.setChecked(True)
        self.pushButton_1.clicked.connect(self.dataprocess)
        self.pushButton_2.clicked.connect(self.result)
        self.pushButton_3.clicked.connect(self.save)
        
        
    def initialize(self,df):
        # 设置表格行数和列数
        columns = df.columns
        rows = df.index
        self.tableWidget.setRowCount(len(rows))
        self.tableWidget.setColumnCount(len(columns))
        self.tableWidget.setHorizontalHeaderLabels(columns)
        for j in range(len(columns)):
            self.tableWidget.setColumnWidth(j, 100)
            for i in range(len(rows)):
                data = QTableWidgetItem(str(df.iloc[i][j]))
                self.tableWidget.setItem(i, j, data)
        self.tableWidget.setAlternatingRowColors(True)
    
    def final(self,df):
        columns = df.columns
        rows = df.index
        self.tableWidget_2.setRowCount(len(rows))
        self.tableWidget_2.setColumnCount(len(columns))
        self.tableWidget_2.setHorizontalHeaderLabels(columns)
        for j in range(len(columns)):
            self.tableWidget_2.setColumnWidth(j, 100)
            for i in range(len(rows)):
                data = QTableWidgetItem(str(df.iloc[i][j]))
                self.tableWidget_2.setItem(i, j, data)
        self.tableWidget_2.setAlternatingRowColors(True)
    
    def dataprocess(self):
        try:
            openfile_name = QFileDialog.getOpenFileName(self,'选择文件','','Excel files(*.xlsx , *.xls)')
            global path_openfile_name
            path_openfile_name = openfile_name[0]
            self.df=pd.read_excel(path_openfile_name)
            self.initialize(self.df)
        except:
            self.printf('Error:数据读取有误，请检查数据。')
        
    def result(self):
        data = self.df.copy()
        try:
            self.input_cols = list(self.df.columns.values[1:int(self.lineEdit.text())+1])
            self.output_cols = list(self.df.columns.values[int(self.lineEdit.text())+1:int(self.lineEdit.text())+int(self.lineEdit_2.text())+1])
            if self.lineEdit_3.text() == '若无可不填写':
                self.un_output_cols = []
            else:
                self.un_output_cols = list(self.df.columns.values[int(self.lineEdit.text())+int(self.lineEdit_2.text())+1:
                    int(self.lineEdit.text())+int(self.lineEdit_2.text())+int(self.lineEdit_3.text())+1])
            self.printf('-------------------------------------')
            self.printf('投入为{}'.format(self.input_cols))
            self.printf('产出为{}'.format(self.output_cols))
            self.printf('非期望产出为{}'.format(self.un_output_cols))
            self.key = self.comboBox.currentText() 
            vv = self.comboBox_3.currentText() 
            dd = self.comboBox_2.currentText()
            self.printf('正在计算结果，请稍后')
            if self.radioButton.isChecked():
                if self.key == 'CCR模型':
                    if self.un_output_cols == []:
                        self.printf('您选择的模型是CCR模型 没有非期望产出')
                    else:
                        self.printf('您选择的模型是CCR模型 有非期望产出')
                    model = MCCR(data=data, input_cols = self.input_cols, output_cols = self.output_cols,un_output_cols = self.un_output_cols, vv = vv )
                    self.res = model.res()
                elif self.key == 'DDF模型':
                    if self.un_output_cols == []:
                        self.printf('您选择的模型是DDF模型,没有非期望产出')
                    else:
                        self.printf('您选择的模型是DDF模型,有非期望产出')
                    model = MDDF(data=data, input_cols = self.input_cols, output_cols = self.output_cols,un_output_cols = self.un_output_cols, vv = vv, dd = dd)
                    self.res = model.res()
                elif self.key == 'SBM模型':
                    if self.un_output_cols == []:
                        self.printf('您选择的模型是SBM模型,没有非期望产出')
                    else:
                        self.printf('您选择的模型是SBM模型,有非期望产出')
                    model = MSBM(data=data, input_cols = self.input_cols, output_cols = self.output_cols,un_output_cols = self.un_output_cols, vv = vv )
                    self.res = model.res()
                self.printf('计算已结束')
                self.printf('-------------------------------------')
                self.final(self.res)
            else:
                if self.radioButton_2.isChecked():
                    status = 'cplex'
                elif self.radioButton_3.isChecked():
                    status = 'glpk'
                self.printf('您选择的求解器为{}'.format(status))
                if self.key == 'CCR模型':
                    if self.un_output_cols == []:
                        self.printf('您选择的模型是CCR模型,没有非期望产出')
                    else:
                        self.printf('您选择的模型是CCR模型,有非期望产出')
                    model = MCCR_pyomo(data=data, input_cols = self.input_cols, output_cols = self.output_cols,un_output_cols = self.un_output_cols, vv = vv, solver = status)
                    self.res = model.res()
                elif self.key == 'DDF模型':
                    if self.un_output_cols == []:
                        self.printf('您选择的模型是DDF模型,没有非期望产出')
                    else:
                        self.printf('您选择的模型是DDF模型,有非期望产出')
                    model = MDDF_pyomo(data=data, input_cols = self.input_cols, output_cols = self.output_cols,un_output_cols = self.un_output_cols, vv = vv, dd = dd, solver = status)
                    self.res = model.res()
                elif self.key == 'SBM模型':
                    if self.un_output_cols == []:
                        self.printf('您选择的模型是SBM模型,没有非期望产出')
                    else:
                        self.printf('您选择的模型是SBM模型,有非期望产出')
                    model = MSBM_pyomo(data=data, input_cols = self.input_cols, output_cols = self.output_cols,un_output_cols = self.un_output_cols, vv = vv, solver = status )
                    self.res = model.res()
                self.printf('计算已结束')
                self.printf('-------------------------------------')
                self.final(self.res)
        except:
            self.printf('出错，请检查步骤是否正确')
            self.printf('可能的原因有:')
            self.printf('1.数据格式设置有误,请参照正确设置方式;')
            self.printf('2.您的系统没有该求解器,请重新选取。')
            self.printf('-------------------------------------')
    
    def save(self):
        path = os.path.split(path_openfile_name)[0]
        self.res.to_excel(path+r'\结果_{}.xlsx'.format(self.key), index=False)
        
# 运行界面
    def printf(self, mes):
        self.textBrowser.append(mes)  # 在指定的区域显示提示信息
        self.cursot = self.textBrowser.textCursor()
        self.textBrowser.moveCursor(self.cursot.End)


# platforms: 
# advanced: 0
# suffix: 
# license: default
def protect_pytransform():

    import pytransform as pytransform

    def assert_builtin(func):
        type = ''.__class__.__class__
        builtin_function = type(''.join)
        if type(func) is not builtin_function:
            raise RuntimeError('%s() is not a builtin' % func.__name__)

    def check_obfuscated_script():
        CO_SIZES = 55, 52, 49, 46, 42, 40, 38, 36
        CO_NAMES = set(['pytransform', 'pyarmor_runtime',
                        '__pyarmor__', '__name__', '__file__'])
        co = pytransform.sys._getframe(3).f_code
        if not ((set(co.co_names) <= CO_NAMES)
                and (len(co.co_code) in CO_SIZES)):
            raise RuntimeError('unexpected obfuscated script')

    def check_mod_pytransform():
        def _check_co_key(co, v):
            return (len(co.co_names), len(co.co_consts), len(co.co_code)) == v
        for k, (v1, v2, v3) in [('dllmethod', ((0, 3, 16), (0, 1, 10), None)), ('init_pytransform', ((0, 1, 10), None, None)), ('init_runtime', ((0, 1, 10), None, None)), ('_load_library', ((33, 22, 700), None, None)), ('get_registration_code', ((0, 1, 10), None, None)), ('get_expired_days', ((0, 1, 10), None, None)), ('get_hd_info', ((12, 10, 124), None, None)), ('get_license_info', ((11, 24, 402), None, None)), ('get_license_code', ((1, 2, 10), None, None)), ('format_platform', ((18, 18, 288), None, None)), ('pyarmor_init', ((3, 1, 22), None, None)), ('pyarmor_runtime', ((12, 5, 128), None, None)), ('_match_features', ((1, 2, 32), None, None))]:
            co = getattr(pytransform, k).__code__
            if not _check_co_key(co, v1):
                raise RuntimeError('unexpected pytransform.py')
            if v2:
                if not _check_co_key(co.co_consts[1], v2):
                    raise RuntimeError('unexpected pytransform.py')
            if v3:
                if not _check_co_key(co.__closure__[0].cell_contents.__code__, v3):
                    raise RuntimeError('unexpected pytransform.py')

    def check_lib_pytransform():
        platname = pytransform.sys.platform
        if platname.startswith('darwin'):
            return
        libname = '_pytransform.dylib' if platname.startswith('darwin') else \
                  '_pytransform.dll' if platname.startswith('win') else \
                  '_pytransform.dll' if platname.startswith('cygwin') else \
                  '_pytransform.so'
        if getattr(pytransform.sys, 'frozen', False):
            filename = pytransform.os.path.join(pytransform.sys._MEIPASS, libname)
        else:
            filename = pytransform.os.path.join(pytransform.os.path.dirname(pytransform.__file__), '', libname)

        with open(filename, 'rb') as f:
            buf = bytearray(f.read())
        value = sum(buf)
        if getattr(pytransform.sys, 'frozen', False) and sys.platform == 'darwin':
            if '':
                value += 886 - sum(b'') + 299
            else:
                value += 1217

        if value not in [120313344]:
            raise RuntimeError('unexpected %s' % filename)

    assert_builtin(sum)
    assert_builtin(open)
    assert_builtin(len)

    check_obfuscated_script()
    check_mod_pytransform()
    check_lib_pytransform()


protect_pytransform()



from gurobipy import *
import pandas as pd
import numpy as np
class MSBM():
    def __init__(self, data: pd.DataFrame, input_cols: list, output_cols: list, un_output_cols: list, vv):
        self.data = data
        self.N = list(range(0,data.shape[0]))
        self.input = input_cols
        self.output = output_cols
        self.undes = un_output_cols
        self.key = vv
        
    def res(self):
        if self.undes == []:
            return self.ress()
        else:
            return self.ress_u()
        
    def ress_u(self):
        efficiencies_SBM_model, x_slacks, y_slacks, u_slacks = self.__SBMmodel_u(self.data)
        df_1 = pd.DataFrame(x_slacks)
        df_2 = pd.DataFrame(y_slacks)
        df_3 = pd.DataFrame(u_slacks)
        df = pd.concat([df_1,df_2,df_3],axis=1)
        df.insert(0,'DMU',self.data.iloc[:,0])
        df.insert(1,'效率',efficiencies_SBM_model)    
        return df
    
    def ress(self):
        efficiencies_SBM_model, x_slacks, y_slacks = self.__SBM_model(self.data)
        df_1 = pd.DataFrame(x_slacks)
        df_2 = pd.DataFrame(y_slacks)
        df = pd.concat([df_1,df_2],axis=1)
        df.insert(0,'DMU',self.data.iloc[:,0])
        df.insert(1,'效率',efficiencies_SBM_model)
        return df
    
    def __SBMmodel_u(self, data):
        efficiencies_SBM_model=[]
        x_slacks = {p: [] for p in self.input}
        y_slacks= {q: [] for q in self.output}
        u_slacks = {u: [] for u in self.undes}
        for i in self.N:
            model = Model('SBMu_model')
            model.Params.LogToConsole = 0
            lambdas = model.addVars(self.N,lb=0,vtype=GRB.CONTINUOUS,name='lambdas')
            slacks_input = model.addVars(self.input,lb=0.0,vtype=GRB.CONTINUOUS, name='slacks_input')
            slacks_output = model.addVars(self.output,lb=0.0,vtype=GRB.CONTINUOUS, name='slacks_output')
            slacks_undes = model.addVars(self.undes,lb=0.0,vtype=GRB.CONTINUOUS, name='slacks_undes')
            t =  model.addVar(lb=0.0,vtype=GRB.CONTINUOUS, name='t')
            model.addConstrs((quicksum(lambdas[j] * data.loc[j,p] for j in self.N) + slacks_input[p] -  t * data.loc[i,p] == 0 for p in self.input))
            model.addConstrs((quicksum(lambdas[j] * data.loc[j,q] for j in self.N) - slacks_output[q]-  t * data.loc[i,q] == 0 for q in self.output))
            model.addConstrs((quicksum(lambdas[j] * data.loc[j,u] for j in self.N) + slacks_undes[u]-  t * data.loc[i,u] == 0 for u in self.undes))
            model.addConstr(t + 1/(len(self.output)+ len(self.undes))* (quicksum(slacks_output[q] / data.loc[i,q] for q in self.output) + quicksum(slacks_undes[u] / data.loc[i,u] for u in self.undes)) == 1)
            if self.key == '规模报酬可变VRS':
                model.addConstr(quicksum(lambdas[j] for j in self.N) == t)
            model.setObjective(t - 1/len(self.input) * quicksum(slacks_input[p] / data.loc[i,p] for p in self.input))
            model.optimize()
            eff = model.getObjective().getValue()
            x_slack = {p: slacks_input[p].x / t.x if np.abs(slacks_input[p].x / t.x) >= 1e-7 else 0 for p in self.input}
            y_slack = {q: slacks_output[q].x / t.x if np.abs(slacks_output[q].x / t.x) >= 1e-7 else 0 for q in self.output}
            u_slack = {u: slacks_undes[u].x / t.x if np.abs(slacks_undes[u].x / t.x) >= 1e-7 else 0 for u in self.undes}
            for s_input in self.input:
                x_slacks[s_input].append(x_slack[s_input])
            for s_output in self.output:
                y_slacks[s_output].append(y_slack[s_output])
            for s_undes in self.undes:
                u_slacks[s_undes].append(u_slack[s_undes])
            efficiencies_SBM_model.append(eff) 
        return efficiencies_SBM_model, x_slacks, y_slacks, u_slacks
    
    def __SBM_model(self,data):
        efficiencies_SBM_model=[]
        x_slacks = {i: [] for i in self.input}
        y_slacks= {j: [] for j in self.output}
        for i in self.N:
            model = Model('SBM_model')
            model.Params.LogToConsole = 0
            lambdas = model.addVars(self.N,lb=0,vtype=GRB.CONTINUOUS,name='lambdas')
            slacks_input = model.addVars(self.input,lb=0.0,vtype=GRB.CONTINUOUS, name='slacks_input')
            slacks_output = model.addVars(self.output,lb=0.0,vtype=GRB.CONTINUOUS, name='slacks_output')
            t =  model.addVar(lb=0.0,vtype=GRB.CONTINUOUS, name='t')
            model.addConstrs((quicksum(lambdas[j] * data.loc[j,p] for j in self.N) + slacks_input[p] -  t * data.loc[i,p] == 0 for p in self.input))
            model.addConstrs((quicksum(lambdas[j] * data.loc[j,q] for j in self.N) - slacks_output[q]-  t * data.loc[i,q] == 0 for q in self.output))
            model.addConstr(t + (1/len(self.output)) * quicksum(slacks_output[q] / data.loc[i,q] for q in self.output) == 1)
            if self.key == '规模报酬可变VRS':
                model.addConstr(quicksum(lambdas[j] for j in self.N) == t)
            model.setObjective(t - 1/len(self.input) * quicksum(slacks_input[p] / data.loc[i,p] for p in self.input))
            model.optimize()
            eff = model.getObjective().getValue()
            x_slack = {p: slacks_input[p].x / t.x if np.abs(slacks_input[p].x / t.x) >= 1e-7 else 0 for p in self.input}
            y_slack = {q: slacks_output[q].x / t.x if np.abs(slacks_output[q].x / t.x) >= 1e-7 else 0 for q in self.output}
            for s_input in self.input:
                x_slacks[s_input].append(x_slack[s_input])
            for s_output in self.output:
                y_slacks[s_output].append(y_slack[s_output])       
            efficiencies_SBM_model.append(eff) 
        return efficiencies_SBM_model, x_slacks, y_slacks


from gurobipy import *
import pandas as pd
class MCCR():
    def __init__(self, data: pd.DataFrame, input_cols: list, output_cols: list,un_output_cols: list, vv):
        self.data = data
        self.K = list(range(0,data.shape[0]))
        self.input = input_cols+un_output_cols
        self.output = output_cols

    def res(self):
        TE = []
        PTE = []
        SE = []
        Type = []
        for i in self.K:
            eff_C, Types = self.__CCRmodel(i)
            eff_B = self.__BCCmodel(i)
            eff_S = eff_C/eff_B
            TE.append(eff_C)
            PTE.append(eff_B)
            SE.append(eff_S)
            Type.append(Types)
        self.data['TE'] = TE
        self.data['PTE'] = PTE
        self.data['SE'] = SE
        self.data['规模报酬分析'] = Type
        return self.data[[self.data.columns.values[0]]+self.input+self.output+['TE','PTE','SE','规模报酬分析']]
    
    def __CCRmodel(self, i):
        model = Model('CCR')
        model.Params.LogToConsole = 0
        model.Params.LogFile= 'result.log'
        lambdas = model.addVars(self.K, lb=0, vtype=GRB.CONTINUOUS, name='lambdas')
        sita = model.addVar(vtype = GRB.CONTINUOUS, name = 'sita')
        model.addConstrs(quicksum(lambdas[j] * self.data.loc[j,p] for j in self.K) - self.data.loc[i,p] >=0 for p in self.output)
        model.addConstrs(quicksum(lambdas[j] * self.data.loc[j,q] for j in self.K) - sita * self.data.loc[i,q] <=0 for q in self.input)
        model.setObjective(sita, GRB.MINIMIZE)
        model.optimize()
        eff = model.getObjective().getValue() if model.status == gurobipy.GRB.Status.OPTIMAL else 'N/A'
        Type = '规模报酬固定' if lambdas.sum().getValue() == 1 else '规模报酬递增' if lambdas.sum().getValue() < 1 else '规模报酬递减'
        return eff, Type
    
    def __BCCmodel(self, i):
        model = Model('BCC')
        model.Params.LogToConsole = 0
        lambdas = model.addVars(self.K, lb=0, vtype=GRB.CONTINUOUS, name='lambdas')
        sita = model.addVar(vtype = GRB.CONTINUOUS, name = 'sita')
        model.addConstrs(quicksum(lambdas[j] * self.data.loc[j,p] for j in self.K) - self.data.loc[i,p] >=0 for p in self.output)
        model.addConstrs(quicksum(lambdas[j] * self.data.loc[j,q] for j in self.K) - sita * self.data.loc[i,q] <=0 for q in self.input)
        model.addConstr(quicksum(lambdas[j] for j in self.K) == 1)
        model.setObjective(sita, GRB.MINIMIZE)
        model.optimize()
        eff = model.getObjective().getValue() if model.status == gurobipy.GRB.Status.OPTIMAL else 'N/A'
        return eff


from pyomo.environ import *  
import pandas as pd
import numpy as np

class MSBM_pyomo():
    def __init__(self, data: pd.DataFrame, input_cols: list, output_cols: list, un_output_cols: list, vv, solver):
        self.data = data
        self.N = list(range(0,data.shape[0]))
        self.input = input_cols
        self.output = output_cols
        self.undes = un_output_cols
        self.key = vv
        self.solvers = solver
        
    def res(self):
        if self.undes == []:
            return self.ress()
        else:
            return self.ress_u()
        
    def ress_u(self):
        efficiencies_SBM_model, x_slacks, y_slacks, u_slacks = self.__SBMmodel_u(self.data)
        df_1 = pd.DataFrame(x_slacks)
        df_2 = pd.DataFrame(y_slacks)
        df_3 = pd.DataFrame(u_slacks)
        df = pd.concat([df_1,df_2,df_3],axis=1)
        df.insert(0,'DMU',self.data.iloc[:,0])
        df.insert(1,'效率',efficiencies_SBM_model)    
        return df
    
    def ress(self):
        efficiencies_SBM_model, x_slacks, y_slacks = self.__SBM_model(self.data)
        df_1 = pd.DataFrame(x_slacks)
        df_2 = pd.DataFrame(y_slacks)
        df = pd.concat([df_1,df_2],axis=1)
        df.insert(0,'DMU',self.data.iloc[:,0])
        df.insert(1,'效率',efficiencies_SBM_model)
        return df
    
    def __SBMmodel_u(self, data):
        efficiencies_SBM_model=[]
        x_slacks = {p: [] for p in self.input}
        y_slacks= {q: [] for q in self.output}
        u_slacks = {u: [] for u in self.undes}
        for i in self.N:
            model = ConcreteModel()
            model.lambdas = Var(self.N, within=NonNegativeReals)
            model.slacks_input = Var(self.input, within=NonNegativeReals)
            model.slacks_output = Var(self.output, within=NonNegativeReals)
            model.slacks_undes = Var(self.undes, within=NonNegativeReals)
            model.t =  Var(within=NonNegativeReals)
            model.constraints = ConstraintList()
            for p in self.input:
                model.constraints.add(expr=sum(model.lambdas[j] * data.loc[j,p] for j in self.N) + model.slacks_input[p] -  model.t * data.loc[i,p] == 0)
            for q in self.output:
                model.constraints.add(expr=sum(model.lambdas[j] * data.loc[j,q] for j in self.N) - model.slacks_output[q]-  model.t * data.loc[i,q] == 0)
            for u in self.undes:
                model.constraints.add(expr=sum(model.lambdas[j] * data.loc[j,u] for j in self.N) + model.slacks_undes[u]-  model.t * data.loc[i,u] == 0)
            model.constraints.add(expr=model.t + 1/(len(self.output)+ len(self.undes))* (sum(model.slacks_output[q] / data.loc[i,q] for q in self.output) + sum(model.slacks_undes[u] / data.loc[i,u] for u in self.undes)) == 1)
            if self.key == '规模报酬可变VRS':
                model.constraints.add(expr=sum(model.lambdas[j] for j in self.N) == model.t)
            model.obj = Objective(expr=model.t - 1/len(self.input) * sum(model.slacks_input[p] / data.loc[i,p] for p in self.input))
            solver = SolverFactory(self.solvers)
            opt = solver.solve(model)
            eff = value(model.obj)
            x_slack = {p: value(model.slacks_input[p]) / value(model.t) if np.abs(value(model.slacks_input[p]) / value(model.t)) >= 1e-7 else 0 for p in self.input}
            y_slack = {q: value(model.slacks_output[q]) / value(model.t) if np.abs(value(model.slacks_output[q]) / value(model.t)) >= 1e-7 else 0 for q in self.output}
            u_slack = {u: value(model.slacks_undes[u]) / value(model.t) if np.abs(value(model.slacks_undes[u]) / value(model.t)) >= 1e-7 else 0 for u in self.undes}
            for s_input in self.input:
                x_slacks[s_input].append(x_slack[s_input])
            for s_output in self.output:
                y_slacks[s_output].append(y_slack[s_output])
            for s_undes in self.undes:
                u_slacks[s_undes].append(u_slack[s_undes])
            efficiencies_SBM_model.append(eff) 
        return efficiencies_SBM_model, x_slacks, y_slacks, u_slacks
    
    def __SBM_model(self,data):
        efficiencies_SBM_model=[]
        x_slacks = {i: [] for i in self.input}
        y_slacks= {j: [] for j in self.output}
        for i in self.N:
            model = ConcreteModel()
            model.lambdas = Var(self.N, within=NonNegativeReals)
            model.slacks_input = Var(self.input, within=NonNegativeReals)
            model.slacks_output = Var(self.output, within=NonNegativeReals)
            model.t =  Var(within=NonNegativeReals)
            model.constraints = ConstraintList()
            for p in self.input:
                model.constraints.add(expr=sum(model.lambdas[j] * data.loc[j,p] for j in self.N) + model.slacks_input[p] -  model.t * data.loc[i,p] == 0)
            for q in self.output:
                model.constraints.add(expr=sum(model.lambdas[j] * data.loc[j,q] for j in self.N) - model.slacks_output[q]-  model.t * data.loc[i,q] == 0)
            model.constraints.add(expr=model.t + (1/len(self.output)) * sum(model.slacks_output[q] / data.loc[i,q] for q in self.output) == 1)
            if self.key == '规模报酬可变VRS':
                model.constraints.add(expr=sum(model.lambdas[j] for j in self.N) == model.t)
            model.obj = Objective(expr=model.t - 1/len(self.input) * sum(model.slacks_input[p] / data.loc[i,p] for p in self.input))
            solver = SolverFactory(self.solvers)
            opt = solver.solve(model)
            eff = value(model.obj)
            x_slack = {p: value(model.slacks_input[p]) / value(model.t) if np.abs(value(model.slacks_input[p]) / value(model.t)) >= 1e-7 else 0 for p in self.input}
            y_slack = {q: value(model.slacks_output[q]) / value(model.t) if np.abs(value(model.slacks_output[q]) / value(model.t)) >= 1e-7 else 0 for q in self.output}
            for s_input in self.input:
                x_slacks[s_input].append(x_slack[s_input])
            for s_output in self.output:
                y_slacks[s_output].append(y_slack[s_output])       
            efficiencies_SBM_model.append(eff) 
        return efficiencies_SBM_model, x_slacks, y_slacks


from deahub.deasolver import Tables
from PySide2.QtWidgets import *
from PySide2 import QtCore
import sys

def run():
    QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling)
    app = QApplication(sys.argv)
    tables = Tables()
    tables.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
    run()


#!/usr/bin/env python
# coding: utf-8
import pathlib
from setuptools import setup,find_packages
path = pathlib.Path(__file__).parent
README = (path / "README.md").read_text(encoding='utf-8')

setup(
    name='deahub',
    version='1.0.0',
    author='WANGY',
    License = "GNU AGPLv3",
    author_email='wangy_prince@126.com',
    long_description = README,
    long_description_content_type = 'text/markdown',
    url='https://github.com/DEATien',
    description=u'Data envelopment analysis efficiency calculator',
    # packages=['deahub'],
    # package_dir={'deahub': 'dist'},
    # data_files=[('Lib/site-packages/deahub/pytransform', ['dist/pytransform/_pytransform.dll','dist/pytransform/__init__.py'])],
    packages=['deahub'],
    package_dir={'deahub': 'dist'},
    include_package_data = True,
    python_requires="<3.10" ,
    install_requires=['pandas','Pyside2','numpy','openpyxl','pyomo','gurobipy'],
    classifiers={
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS',
    },
    entry_points = {
        "console_scripts":
            "deahub=deahub.__main__:run",
    }

)


# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'table_1.ui'
##
## Created by: Qt User Interface Compiler version 5.15.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        if not Dialog.objectName():
            Dialog.setObjectName(u"Dialog")
        Dialog.resize(970, 759)
        self.formLayout = QFormLayout(Dialog)
        self.formLayout.setObjectName(u"formLayout")
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.pushButton_1 = QPushButton(Dialog)
        self.pushButton_1.setObjectName(u"pushButton_1")
        font = QFont()
        font.setFamily(u"Adobe Devanagari")
        font.setPointSize(9)
        self.pushButton_1.setFont(font)

        self.verticalLayout.addWidget(self.pushButton_1)

        self.label_3 = QLabel(Dialog)
        self.label_3.setObjectName(u"label_3")
        font1 = QFont()
        font1.setFamily(u"Adobe Devanagari")
        font1.setBold(True)
        font1.setWeight(75)
        self.label_3.setFont(font1)

        self.verticalLayout.addWidget(self.label_3)

        self.comboBox = QComboBox(Dialog)
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.setObjectName(u"comboBox")
        font2 = QFont()
        font2.setFamily(u"Times New Roman")
        self.comboBox.setFont(font2)
        self.comboBox.setLayoutDirection(Qt.LeftToRight)
        self.comboBox.setAutoFillBackground(False)

        self.verticalLayout.addWidget(self.comboBox)

        self.label_7 = QLabel(Dialog)
        self.label_7.setObjectName(u"label_7")
        font3 = QFont()
        font3.setFamily(u"Adobe Devanagari")
        font3.setBold(False)
        font3.setWeight(50)
        self.label_7.setFont(font3)

        self.verticalLayout.addWidget(self.label_7)

        self.comboBox_3 = QComboBox(Dialog)
        self.comboBox_3.addItem("")
        self.comboBox_3.addItem("")
        self.comboBox_3.setObjectName(u"comboBox_3")
        self.comboBox_3.setFont(font2)

        self.verticalLayout.addWidget(self.comboBox_3)

        self.label_6 = QLabel(Dialog)
        self.label_6.setObjectName(u"label_6")
        self.label_6.setFont(font3)

        self.verticalLayout.addWidget(self.label_6)

        self.comboBox_2 = QComboBox(Dialog)
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.setObjectName(u"comboBox_2")
        self.comboBox_2.setFont(font2)

        self.verticalLayout.addWidget(self.comboBox_2)

        self.label = QLabel(Dialog)
        self.label.setObjectName(u"label")
        font4 = QFont()
        font4.setFamily(u"Adobe Devanagari")
        font4.setPointSize(9)
        font4.setBold(False)
        font4.setWeight(50)
        self.label.setFont(font4)

        self.verticalLayout.addWidget(self.label)

        self.lineEdit = QLineEdit(Dialog)
        self.lineEdit.setObjectName(u"lineEdit")

        self.verticalLayout.addWidget(self.lineEdit)

        self.label_2 = QLabel(Dialog)
        self.label_2.setObjectName(u"label_2")
        self.label_2.setFont(font4)

        self.verticalLayout.addWidget(self.label_2)

        self.lineEdit_2 = QLineEdit(Dialog)
        self.lineEdit_2.setObjectName(u"lineEdit_2")

        self.verticalLayout.addWidget(self.lineEdit_2)

        self.label_5 = QLabel(Dialog)
        self.label_5.setObjectName(u"label_5")
        self.label_5.setFont(font3)

        self.verticalLayout.addWidget(self.label_5)

        self.lineEdit_3 = QLineEdit(Dialog)
        self.lineEdit_3.setObjectName(u"lineEdit_3")
        font5 = QFont()
        font5.setFamily(u"Adobe Devanagari")
        font5.setItalic(True)
        self.lineEdit_3.setFont(font5)

        self.verticalLayout.addWidget(self.lineEdit_3)


        self.formLayout.setLayout(0, QFormLayout.LabelRole, self.verticalLayout)

        self.tableWidget = QTableWidget(Dialog)
        self.tableWidget.setObjectName(u"tableWidget")

        self.formLayout.setWidget(0, QFormLayout.FieldRole, self.tableWidget)

        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.label_15 = QLabel(Dialog)
        self.label_15.setObjectName(u"label_15")
        self.label_15.setFont(font3)

        self.verticalLayout_2.addWidget(self.label_15)

        self.radioButton = QRadioButton(Dialog)
        self.radioButton.setObjectName(u"radioButton")
        self.radioButton.setFont(font2)

        self.verticalLayout_2.addWidget(self.radioButton)

        self.radioButton_2 = QRadioButton(Dialog)
        self.radioButton_2.setObjectName(u"radioButton_2")
        self.radioButton_2.setFont(font2)

        self.verticalLayout_2.addWidget(self.radioButton_2)

        self.radioButton_3 = QRadioButton(Dialog)
        self.radioButton_3.setObjectName(u"radioButton_3")
        self.radioButton_3.setFont(font2)

        self.verticalLayout_2.addWidget(self.radioButton_3)

        self.pushButton_2 = QPushButton(Dialog)
        self.pushButton_2.setObjectName(u"pushButton_2")
        sizePolicy = QSizePolicy(QSizePolicy.Maximum, QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton_2.sizePolicy().hasHeightForWidth())
        self.pushButton_2.setSizePolicy(sizePolicy)
        self.pushButton_2.setMinimumSize(QSize(140, 40))
        self.pushButton_2.setFont(font)

        self.verticalLayout_2.addWidget(self.pushButton_2)


        self.formLayout.setLayout(1, QFormLayout.LabelRole, self.verticalLayout_2)

        self.textBrowser = QTextBrowser(Dialog)
        self.textBrowser.setObjectName(u"textBrowser")

        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.textBrowser)

        self.label_8 = QLabel(Dialog)
        self.label_8.setObjectName(u"label_8")
        self.label_8.setFont(font1)

        self.formLayout.setWidget(2, QFormLayout.FieldRole, self.label_8)

        self.pushButton_3 = QPushButton(Dialog)
        self.pushButton_3.setObjectName(u"pushButton_3")
        sizePolicy1 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.pushButton_3.sizePolicy().hasHeightForWidth())
        self.pushButton_3.setSizePolicy(sizePolicy1)
        self.pushButton_3.setMinimumSize(QSize(140, 20))
        self.pushButton_3.setFont(font)

        self.formLayout.setWidget(3, QFormLayout.LabelRole, self.pushButton_3)

        self.tableWidget_2 = QTableWidget(Dialog)
        self.tableWidget_2.setObjectName(u"tableWidget_2")
        sizePolicy2 = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Expanding)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.tableWidget_2.sizePolicy().hasHeightForWidth())
        self.tableWidget_2.setSizePolicy(sizePolicy2)
        self.tableWidget_2.setMinimumSize(QSize(0, 150))

        self.formLayout.setWidget(3, QFormLayout.FieldRole, self.tableWidget_2)

        self.label_11 = QLabel(Dialog)
        self.label_11.setObjectName(u"label_11")

        self.formLayout.setWidget(4, QFormLayout.FieldRole, self.label_11)

        self.verticalLayout_4 = QVBoxLayout()
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.label_9 = QLabel(Dialog)
        self.label_9.setObjectName(u"label_9")

        self.verticalLayout_4.addWidget(self.label_9)

        self.label_10 = QLabel(Dialog)
        self.label_10.setObjectName(u"label_10")

        self.verticalLayout_4.addWidget(self.label_10)

        self.label_12 = QLabel(Dialog)
        self.label_12.setObjectName(u"label_12")

        self.verticalLayout_4.addWidget(self.label_12)

        self.label_13 = QLabel(Dialog)
        self.label_13.setObjectName(u"label_13")

        self.verticalLayout_4.addWidget(self.label_13)

        self.label_14 = QLabel(Dialog)
        self.label_14.setObjectName(u"label_14")

        self.verticalLayout_4.addWidget(self.label_14)


        self.formLayout.setLayout(5, QFormLayout.FieldRole, self.verticalLayout_4)

        self.label_4 = QLabel(Dialog)
        self.label_4.setObjectName(u"label_4")
        self.label_4.setFont(font2)

        self.formLayout.setWidget(6, QFormLayout.LabelRole, self.label_4)

        self.label_16 = QLabel(Dialog)
        self.label_16.setObjectName(u"label_16")

        self.formLayout.setWidget(6, QFormLayout.FieldRole, self.label_16)


        self.retranslateUi(Dialog)

        QMetaObject.connectSlotsByName(Dialog)
    # setupUi

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QCoreApplication.translate("Dialog", u"Data envelopment analysis 效率计算器  v1.0.1", None))
        self.pushButton_1.setText(QCoreApplication.translate("Dialog", u"\u5bfc\u5165\u8868\u683c", None))
        self.label_3.setText(QCoreApplication.translate("Dialog", u"\u8bf7\u9009\u62e9\u6a21\u578b\uff1a", None))
        self.comboBox.setItemText(0, QCoreApplication.translate("Dialog", u"CCR\u6a21\u578b", None))
        self.comboBox.setItemText(1, QCoreApplication.translate("Dialog", u"SBM\u6a21\u578b", None))
        self.comboBox.setItemText(2, QCoreApplication.translate("Dialog", u"DDF\u6a21\u578b", None))

        self.label_7.setText(QCoreApplication.translate("Dialog", u"\u8bf7\u9009\u62e9\u89c4\u6a21\u62a5\u916c\u6027\uff1a", None))
        self.comboBox_3.setItemText(0, QCoreApplication.translate("Dialog", u"\u89c4\u6a21\u62a5\u916c\u4e0d\u53d8CRS", None))
        self.comboBox_3.setItemText(1, QCoreApplication.translate("Dialog", u"\u89c4\u6a21\u62a5\u916c\u53ef\u53d8VRS", None))

        self.label_6.setText(QCoreApplication.translate("Dialog", u"\u8bf7\u9009\u62e9\u65b9\u5411(DDF\u6a21\u578b)\uff1a", None))
        self.comboBox_2.setItemText(0, QCoreApplication.translate("Dialog", u"\u65e0\u65b9\u5411(\u9664DDF\u6a21\u578b\u5916)", None))
        self.comboBox_2.setItemText(1, QCoreApplication.translate("Dialog", u"(-x,y,-b)\u5f31\u53ef\u5904\u7f6e\u6027=", None))
        self.comboBox_2.setItemText(2, QCoreApplication.translate("Dialog", u"(-x,y,-b)\u5f3a\u53ef\u5904\u7f6e\u6027>=", None))
        self.comboBox_2.setItemText(3, QCoreApplication.translate("Dialog", u"(-x,y,b)\u5f3a\u53ef\u5904\u7f6e\u6027<=", None))

        self.label.setText(QCoreApplication.translate("Dialog", u"\u6295\u5165\u53d8\u91cf\u7684\u4e2a\u6570\uff1a", None))
        self.label_2.setText(QCoreApplication.translate("Dialog", u"\u4ea7\u51fa\u53d8\u91cf\u7684\u4e2a\u6570\uff1a", None))
        self.label_5.setText(QCoreApplication.translate("Dialog", u"\u975e\u671f\u671b\u4ea7\u51fa\u53d8\u91cf\u7684\u4e2a\u6570:", None))
        self.lineEdit_3.setText(QCoreApplication.translate("Dialog", u"\u82e5\u65e0\u53ef\u4e0d\u586b\u5199", None))
        self.label_15.setText(QCoreApplication.translate("Dialog", u"\u6c42\u89e3\u5668\u9009\u53d6\uff1a", None))
        self.radioButton.setText(QCoreApplication.translate("Dialog", u"Gurobi", None))
        self.radioButton_2.setText(QCoreApplication.translate("Dialog", u"Cplex", None))
        self.radioButton_3.setText(QCoreApplication.translate("Dialog", u"Glpk", None))
        self.pushButton_2.setText(QCoreApplication.translate("Dialog", u"\u5f00\u59cb\u8ba1\u7b97", None))
        self.label_8.setText(QCoreApplication.translate("Dialog", u"\u7ed3\u679c\u8f93\u51fa\u8868\u683c\u5c55\u793a\uff1a", None))
        self.pushButton_3.setText(QCoreApplication.translate("Dialog", u"\u7ed3\u679c\u4fdd\u5b58", None))
        self.label_11.setText(QCoreApplication.translate("Dialog", u"* DDF\u6a21\u578b\u9700\u8981\u9009\u62e9\u65b9\u5411\u5411\u91cf\uff0c\u5176\u4ed6\u6a21\u578b\u65e0\u9700\u9009\u62e9\uff0c\u9ed8\u8ba4\u5373\u53ef", None))
        self.label_9.setText(QCoreApplication.translate("Dialog", u"\u6ce8\uff1a", None))
        self.label_10.setText(QCoreApplication.translate("Dialog", u"1. \u5bfc\u5165\u6570\u636e\u65f6\uff0c\u8bf7\u6ce8\u610f\u6570\u636e\u683c\u5f0f\u9700\u6309\u7167DMU\u540d\u3001\u6295\u5165\u3001\u4ea7\u51fa\u7684\u987a\u5e8f\u3002", None))
        self.label_12.setText(QCoreApplication.translate("Dialog", u"2. CCR\u6a21\u578b\u9ed8\u8ba4\u4f1a\u8f93\u51faCRS\u4e0eVRS\u7684\u7ed3\u679c\uff0c\u5e76\u7ed9\u51fa\u89c4\u6a21\u53d8\u5316\u60c5\u51b5\uff1bSBM\u6a21\u578b\u4f1a\u8f93\u51fa\u677e\u5f1b\u53d8\u91cf\u6539\u8fdb\u503c\u3002", None))
        self.label_13.setText(QCoreApplication.translate("Dialog", u"3. \u9700\u8981\u9009\u62e9\u53d8\u91cf\u4e2a\u6570\uff0c\u53ea\u9700\u586b\u5199\u963f\u62c9\u4f2f\u6570\u5b57\uff0c\u5982\u6295\u51652\u4e2a\uff0c\u53ea\u9700\u8981\u5728\u7a7a\u683c\u5185\u586b\u51992\u3002", None))
        self.label_14.setText(QCoreApplication.translate("Dialog", u"4. \u82e5\u6709\u975e\u671f\u671b\u4ea7\u51fa\uff0c\u9700\u5220\u9664\u201c\u82e5\u65e0\u53ef\u4e0d\u586b\u5199\u201d\u540e\u518d\u586b\u5199\u6570\u91cf\u3002", None))
        self.label_4.setText(QCoreApplication.translate("Dialog", u"Design by WANGY", None))
        self.label_16.setText(QCoreApplication.translate("Dialog", u"<html><head/><body><p><a href=\"https://deatien.github.io/\"><span style=\" text-decoration: underline; color:#0000ff;\">请点击获取说明文档</span></a></p></body></html>", None))
        self.label_16.setOpenExternalLinks(True)
    # retranslateUi

